// Copyright (C) 1998 Logi Ragnarsson

package org.logi.crypto.test;
import org.logi.crypto.*;
import org.logi.crypto.keys.*;
import org.logi.crypto.modes.*;

import java.util.Random;

/**
 * This application tests the various encryption-mode classes.
 *
 * @author <a href="http://www.logi.org/~logir/">Logi Ragnarsson</a>
 * (<a href="mailto:logir@logi.org">logir@logi.org</a>)
 *
 * @see org.logi.crypto.modes.EncryptMode
 */
public class TestMode extends Crypto{
    
    private TestMode(){}
    
    // Insecure but fast pseud-random number generator
    private static Random rand=new Random();  

    private static void help(){
        System.out.println("Use: java org.logi.crypto.test.TestKey <key-type> <mode>");
        System.out.println("where <key-type> is one of DES, TriDES, Blowfish, RSA or Caesar.");
        System.out.println("and   <mode> is one of ECB, CBC, CFB or OFB.");
    }
    
    private static KeyPair createKeys(String[] arg){
        if(arg.length==0){
            help();
            return null;
        }
        System.out.println("KEY GENERATION");
        
        if(arg[0].equals("DES")){
            Key k = new DESKey();
            return new KeyPair(k,k);
        } else if(arg[0].equals("TriDES")){
            Key k = new TriDESKey();
            return new KeyPair(k,k);
        } else if(arg[0].equals("Blowfish")){
            Key k = new BlowfishKey();
            return new KeyPair(k,k);
        } else if(arg[0].equals("RSA")){
            return RSAKey.createKeys(256);
        } else if(arg[0].equals("Caesar")){
            Key k = new CaesarKey();
            return new KeyPair(k,k);
        }
        
        help();
        return null;
    }
    
    private static EncryptMode createEncrypt(String[] arg, CipherKey pub, CipherKey pri){
        if(arg.length<2){
            help();
        } else if (arg[1].equals("ECB")){
            return new EncryptECB(pub);
        } else if (arg[1].equals("CBC")){
            return new EncryptCBC(pub);
        } else if (arg[1].equals("CFB")){
            return new EncryptCFB(pri);
        } else if (arg[1].equals("OFB")){
            return new EncryptOFB(pri,256);
        } else
            help();
        
        return null;
    }
    
    private static DecryptMode createDecrypt(String[] arg, CipherKey pub, CipherKey pri){
        if(arg.length<2){
            help();
        } else if (arg[1].equals("ECB")){
            return new DecryptECB(pri);
        } else if (arg[1].equals("CBC")){
            return new DecryptCBC(pri);
        } else if (arg[1].equals("CFB")){
            return new DecryptCFB(pri);
        } else if (arg[1].equals("OFB")){
            return new DecryptOFB(pri,256);
        } else
            help();
        
        return null;
    }
    
    private static boolean test(EncryptMode em, DecryptMode dm){
        long start=System.currentTimeMillis();
        System.out.println("\nTESTING MODE");
        Random rand = new Random();
        
        int SIZE=8192;
        byte[] source = new byte[SIZE];
        byte[] dest   = new byte[SIZE];
        int i=0;
        int j=0;
        int k=0;
        rand.nextBytes(source);
        
        System.out.println("Encrypting "+SIZE/1024+" KB.");
        while(j<SIZE){
            byte[] t1;
            if (i<SIZE){
                // there is data left to encrypt
                int n = Math.min(rand.nextInt()%64+64, SIZE-i);
                t1 = em.encrypt(source,i,n);
                i+=n;
            } else
                t1 = em.flush();
            
            byte[] t2 = dm.decrypt(t1,0,t1.length);
            int n=Math.min(t2.length,SIZE-j);
            System.arraycopy(t2,0, dest,j, n);
            j+=n;
            
            while(k<j){
                if (dest[k]!=source[k]){
                    System.err.println("Error at offset "+k);
                    return false;
                }
                k++;
            }
            System.out.print(".");
            System.out.flush();
        }
        long stop=System.currentTimeMillis();
        System.out.println("\nTime="+(stop-start)+"ms");
        return true;
    }
    
    
    /** Run with arguments DES, TriDES, Blowfish, RSA or Caesar and ECB, CBC, CFB or OFB. */
    public static void main(String[] arg){
        Crypto.initRandom();
        
        KeyPair kp = createKeys(arg);
        if (kp==null)
            return;
        
        // For asymmetric ciphers, pub will be a public key. Other wise pub==pri.
        Key pub = kp.getPublic();
        Key pri = kp.getPrivate();
        
        EncryptMode em = createEncrypt(arg,(CipherKey)pub,(CipherKey)pri);
        if (em==null)
            return;
        DecryptMode dm = createDecrypt(arg,(CipherKey)pub,(CipherKey)pri);
        if (em==null)
            return;
        
        boolean ok = test(em,dm);
        
        System.out.println();
        if(ok)
            System.out.println("All tests passed");
        else
            System.out.println("Some tests failed");
    }
    
}
