// Copyright (C) 1998 Logi Ragnarsson

package org.logi.crypto.test;
import org.logi.crypto.*;
import org.logi.crypto.random.*;

import java.util.Random;
import java.security.SecureRandom;
import java.io.*;

/**
 * This application tests the various random number generation classes.
 *
 * @see org.logi.crypto.keys.Key
 * @version 1.0.6
 *
 * @author <a href="http://www.logi.org/~logir/">Logi Ragnarsson</a>
 * (<a href="mailto:logir@logi.org">logir@logi.org</a>)
 */
public class TestRandom extends Crypto{

    private TestRandom(){}
    
    /**
     * Anything written to instances of this class
     * is counted and discarded.
     */
    private static class BitBucket extends FilterOutputStream{
        
        private int count=0;
        
        public BitBucket(){
            super(null);
        }
        
        public void close(){
        }
        
        public void flush(){
        }
        
        public void write(byte[] buf){
            count += buf.length;
        }
        
        public void write(byte[] buf, int off, int len){
            count += len;
        }
        
        public void write(int x){
            count += 1;
        }
        
        public int getCount(){
            return count;
        }
        
    }
    
    private static void printTime(long t){
        System.out.println("Time: " + (t/1000) + "." + (t%1000) + " s");
    }
    
    private static final int K=16;
    
    private static void test(Random r) throws Exception{
        byte[] bits = new byte[1024*K];
        System.out.println("Generating "+K+" KB of random bits");
        long start = System.currentTimeMillis();
        r.nextBytes(bits);
        long end = System.currentTimeMillis();
        printTime(end-start);
        
        // Apparently, compression is not a good randomness test, since the
        // output of java.util.Random can't be compressed eny better than
        // the proper PRNGs. Maybe the table size really needs to go
        // towards infinity for this to work at all? I had better re-read
        // that paper!
        
        BitBucket bitBucket = new BitBucket();
        OutputStream out = new java.util.zip.GZIPOutputStream(bitBucket,K*1024);
        out.write(bits);
        out.close();
        
        int compressed = bitBucket.getCount()-38;
        double ratio = (double)compressed / (K*1024);
        System.out.println("Compressed to " + compressed + " bytes, ratio: " + ratio);
        
        System.out.println();
    }
    
    public static void main(String[] arg){
        long start,end;
        Random r;

        Crypto.initRandom();
       
        try{
            System.out.println("Initializing java.util.Random");
            start = System.currentTimeMillis();
            r = new Random();
            end = System.currentTimeMillis();
            printTime(end-start);
            test(r);
        } catch (Throwable e){
            e.printStackTrace();
        }
        
        try{
            System.out.println("Initializing java.security.SecureRandom");
            start = System.currentTimeMillis();
            r = new SecureRandom();
            end = System.currentTimeMillis();
            printTime(end-start);
            test(r);
        } catch (Throwable e){
            e.printStackTrace();
        }
        
        try{
            System.out.println("Initializing org.logi.crypto.random.RandomMD5");
            start = System.currentTimeMillis();
            r = new RandomMD5();
            int i = r.nextInt();
            end = System.currentTimeMillis();
            printTime(end-start);
            test(r);
        } catch (Throwable e){
            e.printStackTrace();
        }
        
        try{
            System.out.println("Initializing org.logi.crypto.random.PureSpinner");
            System.out.println("(this is not a statistically good RNG,");
            System.out.println(" but collects entropy for use in e.g. RandomMD5)");
            start = System.currentTimeMillis();
            r = new PureSpinner();
            end = System.currentTimeMillis();
            printTime(end-start);
            test(r);
        } catch (Throwable e){
            e.printStackTrace();
        }
        
    }
    
}
